/******************************************************************************/
/* Files to Include                                                           */
/******************************************************************************/

#if defined(__XC)
#include <xc.h>         /* XC8 General Include File */
#elif defined(HI_TECH_C)
#include <htc.h>        /* HiTech General Include File */
#endif

#include <stdint.h>
#include <stdio.h>
#include <pic16f18324.h>         /* For uint8_t definition */

#include "user.h"



/******************************************************************************/
/* User Functions                                                             */

/******************************************************************************/

/**
 * @brief __eeprom_read
 * Read single word from the EEPROM 
 * @param address Address of the cell
 * @return One byte of data
 */
static uint8_t __eeprom_read(uint16_t address) {
    /* Clear the NVMREGS bit of the NVMCON1 register if the 
     * user intends to access Program Flash Memory locations,
     * or set NVMREGS if the user intends to access User ID, Configuration,
     * of EEPROM locations. */
    NVMCON1bits.NVMREGS = 1;

    /* Write the desired address into the NVMADDRH:NVMADDRL register pair. */
    NVMADR = address;

    /* Set the RD bit of the NVMCON1 register to initiate the read. */
    NVMCON1bits.RD = 1;

    /* Return data read */
    return NVMDATL;
}

/**
 * @brief __eeprom_unlock
 * NVM unlock sequence
 */
static inline void __eeprom_unlock(void) {
    /* Write 55h to NVMCON2 */
    NVMCON2bits.NVMCON2 = 0x55;

    /* Write AAh to NVMCON2 */
    NVMCON2bits.NVMCON2 = 0xAA;

    /* Set the WR bit of NVMCON1 */
    NVMCON1bits.WR = 1;
}

/**
 * @brief __eeprom_write
 * Write single byte to EEPROM memory
 * 
 * @param address EEPROM address
 * @param data Value
 */
static void __eeprom_write(uint16_t address, uint8_t data) {
    /* Set the NVMREGS and WREN bits of the NVMCON1 register. */
    NVMCON1bits.NVMREGS = 1;
    NVMCON1bits.WREN = 1;

    /* Write the desired address into NVMADRH:NVMADRL register pair, */
    NVMADR = address;

    /* Load data */
    NVMDATL = data;


    /* Perform the unlock sequence. */
    __eeprom_unlock();


    /* Wait for process to finish. */
    NVMCON1bits.WR = 1;
    while (NVMCON1bits.WR);
    
    /* Disable write */
    NVMCON1bits.WREN = 0;

}
/**
 * @brief read_configuration
 * Read configuration saved in the chip EEPROM
 * @param settings Address of the settings variable
 */
void read_configuration(void *settings) {
    /* Save GIE */
    uint8_t gie = INTCONbits.GIE;

    uint8_t i;
    uint8_t *p = (uint8_t *) (settings);

    for (i = 0; i < sizeof (settings_t); i++) {
        *(p + i) = __eeprom_read(CONFIGS_ADDRESS + i);
    }

    /* Restore GIE */
    INTCONbits.GIE = gie;
}

/**
 * @brief save_configuration
 * Save current settings
 * 
 * @param settings Pointer to settings variable
 */
void save_configuration(const void * const settings) {
    /* Save GIE */
    uint8_t gie = INTCONbits.GIE;

    uint8_t i;
    uint8_t *p = (uint8_t *) (settings);

    for (i = 0; i < sizeof (settings_t); i++) {
        __eeprom_write(CONFIGS_ADDRESS + i, *(p + i));
    }

    /* Restore GIE */
    INTCONbits.GIE = gie;
}

void erase_configuration(void) {
    /* Save GIE */
    uint8_t gie = INTCONbits.GIE;
   
     uint8_t i;
     
     for (i = 0; i < sizeof(settings_t); i++) {
         __eeprom_write(CONFIGS_ADDRESS + i, 0xFF);
     }
     
    /* Restore GIE */
    INTCONbits.GIE = gie;
}