/******************************************************************************/
/*Files to Include                                                            */
/******************************************************************************/

#if defined(__XC)
#include <xc.h>         /* XC8 General Include File */
#elif defined(HI_TECH_C)
#include <htc.h>        /* HiTech General Include File */
#endif

#include <stdint.h>         /* For uint8_t definition */
#include <stdbool.h>        /* For true/false definition */
#include <stdio.h>
#include <pic16f18324.h>

#include "user.h"


/******************************************************************************/
/* Interrupt Routines                                                         */
/******************************************************************************/

static volatile uint8_t __dummy_byte;
static volatile uint8_t __rx_data_count;

static uint8_t *pRegisters;

#ifndef _PIC12

/* Declare methods */
static void inline __handle_ssp_isr(void);
static void inline __handle_bcl_isr(void);
static void inline __handle_rc_isr(void);

/**
 * @brief interrupt isr
 * Main interrupt service routine.
 */
void interrupt isr(void) {

    if (PIR1bits.SSP1IF) {
        __handle_ssp_isr();
    } else if (PIR1bits.BCL1IF) {
        __handle_bcl_isr();
    } else if (PIR1bits.RCIF)
        __handle_rc_isr();
}

static void inline __handle_rc_isr(void) {
    if (RC1STA & 0x06) {
        RC1STAbits.CREN = 0;
        __dummy_byte = RCREG;
        RC1STAbits.CREN = 1;
    } else {
        enqueue(&rx, RCREG);
    }
    PIR1bits.RCIF = 0;
}

/**
 * @brief __handle_bcl_isr
 * Interrupt routine for handling bus collision
 */
static void inline __handle_bcl_isr(void) {
    /* Clear buffer */
    __dummy_byte = SSPBUF;

    /* Clear bus collision interrupt flag */
    PIR1bits.BCL1IF = 0;

    /* Release clock line */
    SSP1CON1bits.CKP = 1;

}

/**
 * @brief __handle_ssp_isr
 * Interrupt routine for handle I2C communication
 */
static void inline __handle_ssp_isr(void) {
    /* Check for read/write */
    if (SSPSTATbits.R_nW) {
        /* We have READ */

        if (!SSPSTATbits.D_nA) {
            /* Last was address */
            __dummy_byte = SSPBUF;

            /* Initialize variables */
            if (pRegisters) {

                if (pRegisters == (uint8_t *) (&registers.rx)) {
                    /* Check if we are gonna read RX_FIFO */
                    if (queue_available(&rx)) {
                        SSPBUF = dequeue(&rx);
                    } else {
                        SSPBUF = 0x00;
                    }
                } else {

                    SSPBUF = *(pRegisters++);
                }
            } else {
                SSPBUF = 0xFF;
            }

            /* Release clock line */
            SSPCON1bits.CKP = 1;
        } else if (!SSP1CON2bits.ACKSTAT) {
            /* Last was data */
            if (pRegisters > (uint8_t *) (&registers.id + sizeof (registers))) {
                pRegisters = NULL;
            }

            __dummy_byte = SSPBUF;
            SSPBUF = 0xFF;

            if (pRegisters) {

                if (pRegisters == (uint8_t *) (&registers.rx)) {
                    /* Check if we are gonna read RX_FIFO */
                    if (queue_available(&rx)) {
                        SSPBUF = dequeue(&rx);
                    } else {
                        SSPBUF = 0x00;
                    }
                } else {

                    SSPBUF = *(pRegisters++);
                }
            }

            /* Release clock line */
            SSPCON1bits.CKP = 1;
        } else {
            SSPBUF = 0xFF;
            /* Release clock line */
            SSPCON1bits.CKP = 1;
        }
    } else {
        /* We have WRITE */
        if (!SSPSTATbits.D_nA) {
            /* Last was address */
            __dummy_byte - SSPBUF;
            __rx_data_count = 0;

            /* Release clock line */
            SSPCON1bits.CKP = 1;
        } else {
            /* Last was data */

            /* We want to read single byte */
            if (__rx_data_count == 0) {

                if (SSPBUF >= REGISTER_ID && SSPBUF < REGISTER_ID + sizeof (registers)) {
                    pRegisters = (uint8_t *) (&registers.id + (SSPBUF - REGISTER_ID));
                } else {
                    pRegisters = NULL;
                }
            } else {
                /* We are going to write multiple bytes */
                if (pRegisters != (uint8_t *) (&registers.id) &&
                        pRegisters != (uint8_t *) (&registers.fw) &&
                        pRegisters != (uint8_t *) (&registers.rx) &&
                        pRegisters != NULL) {

                    /* Check if pointer is outside of register table */
                    if (pRegisters > (uint8_t *) (&registers.id + sizeof (registers))) {
                        pRegisters = NULL;
                        __dummy_byte = SSPBUF;

                    } else {
                        if (pRegisters == (uint8_t *) (&registers.tx)) {
                            enqueue(&tx, SSPBUF);
                        } else {
                            *(pRegisters++) = SSPBUF;
                        }
                    }
                } else {
                    pRegisters++;
                    __dummy_byte = SSPBUF;
                }
            }
            __rx_data_count++;

            /* Check for write collision */
            if (SSP1CON1bits.WCOL) {
                SSP1CON1bits.WCOL = 0;
                __dummy_byte = SSPBUF;
            }

            /* Release clock line */
            SSPCON1bits.CKP = 1;
        }
    }

    /* Clear interrupt flag */
    PIR1bits.SSP1IF = 0;

}
#endif


