/******************************************************************************/
/* Files to Include                                                           */
/******************************************************************************/

#if defined(__XC)
#include <xc.h>         /* XC8 General Include File */
#elif defined(HI_TECH_C)
#include <htc.h>        /* HiTech General Include File */
#endif

#include <stdint.h>        /* For uint8_t definition */
#include <stdbool.h>
#include <stdio.h>
#include <string.h>
#include <pic16f18324.h>


#include "system.h"        /* System funct/params, like osc/peripheral config */
#include "user.h"          /* User funct/params, such as InitApp */

/******************************************************************************/
/* User Global Variable Declaration                                           */
/******************************************************************************/

registers_t registers = {
    DEFAULT_ID, FIRMWARE,
    {
        0xFF, 0xFF, 0xFF, 0xFF
    }
};

settings_t default_settings = {
    DEFAULT_ADDRESS, MODE_CLC, 0x03, UART_B115200
};

settings_t last_settings;

fifo_t rx = {
    -1, -1
};

fifo_t tx = {
    -1, -1
};

/******************************************************************************/
/* Main Program                                                               */

/******************************************************************************/
void main(void) {


    /* Configure the oscillator for the device */
    ConfigureOscillator();

    /* Initialize I/O and Peripherals for application */
    InitApp();

#if defined(__DEBUG__)
    printf("Initialization finish...003\n\r");
#endif

    /* If jumper is closed during power-up sequence, then
     restore default settings.*/
    if (!PORTCbits.RC1) {
        if (memcmp(&registers.current_settings,
                &default_settings, sizeof (settings_t))) {
            /* Set default configuration */
            save_configuration(&default_settings);

#if defined(__DEBUG__)
            printf("Restoring defaults");
#endif

            /* Wait for reset */
            while (1) {
                asm("NOP");
            }

        }
    }

    while (1) {

        /* Check if there are data in the circular buffer */
        if (queue_available(&tx)) {
            uartSend((char) dequeue(&tx));
        }

        /* Check if there are new configurations */
        if (memcmp(&last_settings, &registers.current_settings, sizeof (last_settings))) {

            /* Configure #RE and DE pins */
            LATCbits.LATC2 = !registers.current_settings.clc_control.rx_en;
            LATCbits.LATC3 = registers.current_settings.clc_control.tx_en;
            
            /* Toggle LEDs */
            LATAbits.LATA0 = registers.current_settings.clc_control.tx_en;
            LATAbits.LATA1 = registers.current_settings.clc_control.rx_en;

            if (registers.current_settings.mode == MODE_CLC) {
                CLC1CONbits.LC1EN = registers.current_settings.clc_control.tx_en;
                CLC2CONbits.LC2EN = registers.current_settings.clc_control.rx_en;
            } else {
                TX1STAbits.TXEN = registers.current_settings.clc_control.tx_en;

                if (registers.current_settings.clc_control.rx_en) {
                    RC1STAbits.CREN = 1;
                    PIE1bits.RCIE = 1;
                } else {
                    RC1STAbits.CREN = 0;
                    PIE1bits.RCIE = 0;
                }
            }

            /* Check if we need to update baudrate */
            if (registers.current_settings.baudrate != last_settings.baudrate)
                update_baudrate();

            /* Check if we need to change address */
            if (registers.current_settings.address != last_settings.address) {
                /* This is only available when PROG jumper is closed. */
                if (PORTCbits.RC1) {
                    /* Restore previously state */
                    registers.current_settings.address = last_settings.address;
                } else {
                    /* Apply new address */
                    SSPADD = registers.current_settings.address << 1;
                }
            }

            /* Check if we need to change transmit mode */
            if (registers.current_settings.mode != last_settings.mode) {
                save_configuration(&registers.current_settings);
                while (1) {
                    asm("NOP");
                }
            }

            /* Save modified configuration */
            save_configuration(&registers.current_settings);
            memcpy(&last_settings, &registers.current_settings, sizeof (last_settings));

        }

        /* Clear WDT */
        CLRWDT();
    }

}

